@@ -9,9 +9,10 @@ from datasets import get_dataset_infos
 from pygments import highlight
 from pygments.formatters import HtmlFormatter
 from pygments.lexers import DjangoLexer
+from templates import INCLUDED_USERS
 
 from promptsource.session import _get_state
-from promptsource.templates import Template, TemplateCollection
+from promptsource.templates import DatasetTemplates, Template, TemplateCollection
 from promptsource.utils import (
     get_dataset,
     get_dataset_confs,
@@ -40,10 +41,11 @@ else:
     side_bar_title_prefix = "Promptsource"
 
 #
-# Helper functions for datasets library
+# Cache functions
 #
 get_dataset = st.cache(allow_output_mutation=True)(get_dataset)
 get_dataset_confs = st.cache(get_dataset_confs)
+list_datasets = st.cache(list_datasets)
 
 
 def reset_template_state():
@@ -80,12 +82,21 @@ st.markdown(
     "<style>" + HtmlFormatter(style="friendly").get_style_defs(".highlight") + "</style>", unsafe_allow_html=True
 )
 
-WIDTH = 80
+WIDTH = 140
 
 
 def show_jinja(t, width=WIDTH):
+    def replace_linebreaks(t):
+        """
+        st.write does not handle double breaklines very well. When it encounters `\n\n`, it exit the curent <div> block.
+        Explicitely replacing all `\n` with their html equivalent to bypass this issue.
+        Also stripping the trailing `\n` first.
+        """
+        return t.strip("\n").replace("\n", "<br/>")
+
     wrap = textwrap.fill(t, width=width, replace_whitespace=False)
     out = highlight(wrap, DjangoLexer(), HtmlFormatter())
+    out = replace_linebreaks(out)
     st.write(out, unsafe_allow_html=True)
 
 
@@ -98,27 +109,27 @@ def show_text(t, width=WIDTH, with_markdown=False):
         st.text(wrap)
 
 
-#
-# Loads template data
-#
-try:
-    template_collection = TemplateCollection()
-except FileNotFoundError:
-    st.error(
-        "Unable to find the prompt folder!\n\n"
-        "We expect the folder to be in the working directory. "
-        "You might need to restart the app in the root directory of the repo."
-    )
-    st.stop()
-
-
 if mode == "Helicopter view":
     st.title("High level metrics")
+    st.write("This will take a minute to collect.")
     st.write(
         "If you want to contribute, please refer to the instructions in "
         + "[Contributing](https://github.com/bigscience-workshop/promptsource/blob/main/CONTRIBUTING.md)."
     )
 
+    #
+    # Loads template data
+    #
+    try:
+        template_collection = TemplateCollection()
+    except FileNotFoundError:
+        st.error(
+            "Unable to find the prompt folder!\n\n"
+            "We expect the folder to be in the working directory. "
+            "You might need to restart the app in the root directory of the repo."
+        )
+        st.stop()
+
     #
     # Global metrics
     #
@@ -202,7 +213,7 @@ if mode == "Helicopter view":
     fig.update_xaxes(visible=False, showticklabels=False)
     st.plotly_chart(fig, use_container_width=True)
     st.write(
-        f"- Top 3 training subsets account for `{100*plot_df[:3]['Train size'].sum()/nb_training_instances:.2f}%` of the training instances."
+        f"- Top 3 training subsets account for `{100 * plot_df[:3]['Train size'].sum() / nb_training_instances:.2f}%` of the training instances."
     )
     biggest_training_subset = plot_df.iloc[0]
     st.write(
@@ -228,10 +239,7 @@ else:
     # Loads dataset information
     #
 
-    dataset_list = list_datasets(
-        template_collection,
-        state,
-    )
+    dataset_list = list_datasets()
     ag_news_index = dataset_list.index("ag_news")
 
     #
@@ -258,7 +266,20 @@ else:
         if len(configs) > 0:
             conf_option = st.sidebar.selectbox("Subset", configs, index=0, format_func=lambda a: a.name)
 
-        dataset = get_dataset(dataset_key, str(conf_option.name) if conf_option else None)
+        subset_name = str(conf_option.name) if conf_option else None
+        try:
+            dataset = get_dataset(dataset_key, subset_name)
+        except OSError as e:
+            st.error(
+                f"Some datasets are not handled automatically by `datasets` and require users to download the "
+                f"dataset manually. This applies to {dataset_key}{f'/{subset_name}' if subset_name is not None else ''}. "
+                f"\n\nPlease download the raw dataset to `~/.cache/promptsource/{dataset_key}{f'/{subset_name}' if subset_name is not None else ''}`. "
+                f"\n\nYou can choose another cache directory by overriding `PROMPTSOURCE_MANUAL_DATASET_DIR` environment "
+                f"variable and downloading raw dataset to `$PROMPTSOURCE_MANUAL_DATASET_DIR/{dataset_key}{f'/{subset_name}' if subset_name is not None else ''}`"
+                f"\n\nOriginal error:\n{str(e)}"
+            )
+            st.stop()
+
         splits = list(dataset.keys())
         index = 0
         if "train" in splits:
@@ -267,7 +288,18 @@ else:
         dataset = dataset[split]
         dataset = renameDatasetColumn(dataset)
 
-        dataset_templates = template_collection.get_dataset(dataset_key, conf_option.name if conf_option else None)
+        #
+        # Loads template data
+        #
+        try:
+            dataset_templates = DatasetTemplates(dataset_key, conf_option.name if conf_option else None)
+        except FileNotFoundError:
+            st.error(
+                "Unable to find the prompt folder!\n\n"
+                "We expect the folder to be in the working directory. "
+                "You might need to restart the app in the root directory of the repo."
+            )
+            st.stop()
 
         template_list = dataset_templates.all_template_names
         num_templates = len(template_list)
@@ -315,12 +347,21 @@ else:
         #
         st.header("Dataset: " + dataset_key + " " + (("/ " + conf_option.name) if conf_option else ""))
 
-        st.markdown(
-            "*Homepage*: "
-            + dataset.info.homepage
-            + "\n\n*Dataset*: https://github.com/huggingface/datasets/blob/master/datasets/%s/%s.py"
-            % (dataset_key, dataset_key)
-        )
+        # If we have a custom dataset change the source link to the hub
+        split_dataset_key = dataset_key.split("/")
+        possible_user = split_dataset_key[0]
+        if len(split_dataset_key) > 1 and possible_user in INCLUDED_USERS:
+            source_link = "https://huggingface.co/datasets/%s/blob/main/%s.py" % (
+                dataset_key,
+                split_dataset_key[-1],
+            )
+        else:
+            source_link = "https://github.com/huggingface/datasets/blob/master/datasets/%s/%s.py" % (
+                dataset_key,
+                dataset_key,
+            )
+
+        st.markdown("*Homepage*: " + dataset.info.homepage + "\n\n*Dataset*: " + source_link)
 
         md = """
         %s
@@ -428,7 +469,6 @@ else:
                     state.new_template_name = None
 
             with col1b, st.beta_expander("or Select Prompt", expanded=True):
-                dataset_templates = template_collection.get_dataset(*state.templates_key)
                 template_list = dataset_templates.all_template_names
                 if state.template_name:
                     index = template_list.index(state.template_name)
@@ -578,7 +618,6 @@ else:
                             st.write("Target")
                             show_text(prompt[1], width=40)
 
-
 #
 # Must sync state at end
 #
